#include "stdtypes.h"
#include "reg_tool_3.h"
#include "rt3_redline.h"


#include <OpenGL/gl.h>
#include <string.h>
#include <stdio.h>
#include "gametime.h"
#include "gamemem.h"
#include "fileio.h"
#include "rendercar.h"
#include "carphysics.h"
#include "screen.h"
#include "controls.h"
#include "environment.h"
#include "transparency.h"
#include "renderframe.h"
#include "text.h"
#include "interface.h"
#include "gameinitexit.h"
#include "carselection.h"
#include "config.h"
#include "interfaceutil.h"
#include "stencil.h"
#include "gamesystem.h"
#include "textures.h"
#include "challenges.h"
#include "gamesound.h"
#include "random.h"

//#define REGISTERED (RT3_IsRegistered())
#define REGISTERED true

void GetAvailableCars(tFileRef *availableCars,int *carCount,int onlyBuiltIn,int onlyAvailable)
{
	*carCount=0;
	for(int i=0;i<gFileTableSize;i++)
		if(*carCount<kMaxCars)
			if(char *extension=FileGetExtension(i))
				if(!_stricmp(extension,kFileTypeCarDefinition))
				{
					tCarDefinition *car=(tCarDefinition*)FileGetParsedDataPtr(i,kParserTypeCarDesc,sizeof(tCarDefinition));
					if((car->builtIn||!onlyBuiltIn)&&HasChallengeRequirements(car->challengeRequirements,gConfig->challengeData))
						availableCars[(*carCount)++]=i;
				}
	if(!onlyAvailable)
		for(int i=0;i<gFileTableSize;i++)
			if(*carCount<kMaxCars)
				if(char *extension=FileGetExtension(i))
					if(!_stricmp(extension,kFileTypeCarDefinition))
					{
						tCarDefinition *car=(tCarDefinition*)FileGetParsedDataPtr(i,kParserTypeCarDesc,sizeof(tCarDefinition));
						if((car->builtIn||!onlyBuiltIn)&&!HasChallengeRequirements(car->challengeRequirements,gConfig->challengeData)&&!car->secret)
							availableCars[(*carCount)++]=i;
					}
}

#define kSpecsXPos -0.9
#define kSpecsYPos -0.3

#define kColorsXPos 0.5
#define kColorsYPos 0.2
#define kColorsWidth 0.04
#define kColorsHeight 0.02
#define kColorsOffset 0.025
#define kPointerSize 0.03

void CarSelectionDrawBackground(int mode)
{
	char title[256];
	
	switch(mode)
	{
		case kCarSelectionQuickMode:
			strcpy(title,"Select Car:");
			break;
			
		case kCarSelectionEnemyMode:
			strcpy(title,"Select Opponent Car:");
			break;
		
		case kCarSelectionDisplayMode:
			strcpy(title,"New Car Unlocked!");
			break;
			
	}
	InterfaceDrawBackground(-1,-1,0,0,0,0,1);
	TextPrintfToBufferFormated(Vector(kTitlePosX,kTitlePosY),0.08,kTextAlignLeft,title);
}

void CarSelectionDrawCar(tFileRef carRef,float time,int addOns,int color)
{
	tGameEntity carEntity,cameraEntity;
	tCarPhysics *phys=(tCarPhysics*)MemoryAllocateZeroedBlock(sizeof(tCarPhysics));
	cameraEntity.pos=Vector(0,1,0);
	carEntity.pos=Vector(0,0,7);
	*MatrixGetZVector(cameraEntity.dir)=!(carEntity.pos-cameraEntity.pos);
	*MatrixGetYVector(cameraEntity.dir)=Vector(0,1,0);
	MatrixReAdjust(cameraEntity.dir);
	carEntity.pos.x-=0.7;
	carEntity.pos.y-=1;
	MatrixIdentity(carEntity.dir);
	MatrixRotY(carEntity.dir,time*PI/4);

	carEntity.renderType=kRenderTypeCar;
	carEntity.physicsType=kPhysicsTypeCar;
	carEntity.physicsData=carRef;
	carEntity.physics=phys;

	tCarDefinition *car=(tCarDefinition*)FileGetParsedDataPtr(carRef,kParserTypeCarDesc,sizeof(tCarDefinition));
	phys->car=*car;
	phys->car.wheels=(tWheelDefinition*)MemoryAllocateBlock(sizeof(tWheelDefinition)*car->numWheels);
	MemoryMove(phys->car.wheels,car->wheels,sizeof(tWheelDefinition)*car->numWheels);
	phys->addOns=addOns;
	phys->color=color;
	InstallCarAddOns(phys);
	car=&phys->car;
	
	for(int i=0;i<car->numWheels;i++)
		phys->wheels[i].suspension=car->wheels[i].maxSuspension/2;
		
	tGameEntity *oldCameraEntity=gCameraEntity;
	gCameraEntity=&cameraEntity;
	SetupTranslation(carEntity.pos,carEntity.dir);
	glColorMask(0,0,0,0);
	glBegin(GL_QUADS);
	glVertex3f(-100,0,100);
	glVertex3f(100,0,100);
	glVertex3f(100,0,-100);
	glVertex3f(-100,0,-100);
	glEnd();			
	glColorMask(1,1,1,1);

	carEntity.pos.y=-1-car->wheels->pos.y+car->wheels->maxSuspension/2+car->wheels->radius;	
	SetupTranslation(carEntity.pos,carEntity.dir);

	glPushAttrib(GL_LIGHTING_BIT+GL_POLYGON_BIT+GL_COLOR_BUFFER_BIT+GL_TEXTURE_BIT);
	CarRenderEntity(&carEntity,gConfig->interiorDisplay?true:false,false);
	glPopAttrib();
	
	for(int i=1;i<=gConfig->stencil;i++)
	{
		gStencilZoom=0.9+0.1*i/(float)gConfig->stencil;
		SetupTranslation(carEntity.pos,carEntity.dir);
		glPushAttrib(GL_ENABLE_BIT+GL_POLYGON_BIT);
		CarRenderEntityShadow(&carEntity,6);
		glPopAttrib();
		RenderStencilLayer(true);
	}
	DrawTransparentPolys(NULL);

	MemoryFreeBlock(phys->car.wheels);
	MemoryFreeBlock(phys);

	gCameraEntity=oldCameraEntity;

}

void CarSelectionDrawSpecs(tFileRef carRef,int mode,int *available)
{
	//if(fadeName)
		//gTextOpacity=1;

	tCarDefinition *car=(tCarDefinition*)FileGetParsedDataPtr(carRef,kParserTypeCarDesc,sizeof(tCarDefinition));
	TextPrintfToBufferFormated(Vector(kSpecsXPos,0.5),0.05,kTextAlignLeft,car->carName);
	

//	gTextOpacity=opacity;
	float yPos=kSpecsYPos;
	
	if(car->magic)
	{
		TextPrintfToBufferFormated(Vector(kSpecsXPos,yPos-0.5),0.04,kTextAlignLeft,"Where we're going, we don't need specifications.");
	}
	else
	{		
		if(car->numWheels==4)
			if(car->wheels[0].powered&&car->wheels[2].powered)
				TextPrintfToBufferFormated(Vector(kSpecsXPos,yPos-0.0),0.04,kTextAlignLeft,"Drivetrain: \255#a\2554WD");
			else if(car->wheels[0].powered)
				TextPrintfToBufferFormated(Vector(kSpecsXPos,yPos-0.0),0.04,kTextAlignLeft,"Drivetrain: \255#a\255FWD");
			else if(car->wheels[2].powered)
				TextPrintfToBufferFormated(Vector(kSpecsXPos,yPos-0.0),0.04,kTextAlignLeft,"Drivetrain: \255#a\255RWD");
		
		TextPrintfToBufferFormated(Vector(kSpecsXPos,yPos-0.5),0.04,kTextAlignLeft,"Gearbox: \255#a\255%d Gears",car->numGears-2);
		if(gConfig->metricUnits)
		{
			TextPrintfToBufferFormated(Vector(kSpecsXPos,yPos-0.1),0.04,kTextAlignLeft,"Displacement: \255#a\255%1.1f L",car->displacement);
			TextPrintfToBufferFormated(Vector(kSpecsXPos,yPos-0.2),0.04,kTextAlignLeft,"Mass: \255#a\255%4.0f kg",car->mass);
			TextPrintfToBufferFormated(Vector(kSpecsXPos,yPos-0.3),0.04,kTextAlignLeft,"Power: \255#a\255%3.0f kw @ %4.0f RPM",car->power/1000,car->powerRPM);
			TextPrintfToBufferFormated(Vector(kSpecsXPos,yPos-0.4),0.04,kTextAlignLeft,"Torque: \255#a\255%3.0f Nm @ %4.0f RPM",car->torque,car->torqueRPM);
		}
		else
		{
			TextPrintfToBufferFormated(Vector(kSpecsXPos,yPos-0.1),0.04,kTextAlignLeft,"Displacement: \255#a\255%3.0f cui",car->displacement*61.023744);
			TextPrintfToBufferFormated(Vector(kSpecsXPos,yPos-0.2),0.04,kTextAlignLeft,"Mass: \255#a\255%4.0f lbs",car->mass*2.2046);
			TextPrintfToBufferFormated(Vector(kSpecsXPos,yPos-0.3),0.04,kTextAlignLeft,"Power: \255#a\255%3.0f hp @ %4.0f RPM",car->power/745.69987,car->powerRPM);
			TextPrintfToBufferFormated(Vector(kSpecsXPos,yPos-0.4),0.04,kTextAlignLeft,"Torque: \255#a\255%3.0f lb-ft @ %4.0f RPM",car->torque/1.35628105,car->torqueRPM);
		}	
	}
	if(!car->demoAvailable&&!REGISTERED&&mode!=kCarSelectionEnemyMode)		
		TextPrintfToBufferFormatedColored(Vector(0,0),0.15,kTextAlignMiddle,1,1,1,0.5,"DEMO");
	if(!HasChallengeRequirements(car->challengeRequirements,gConfig->challengeData)&&mode!=kCarSelectionEnemyMode)		
		TextPrintfToBufferFormatedColored(Vector(0.2,0),0.15,kTextAlignMiddle,1,1,1,0.5,"LOCKED");
	if(available)
		*available=((car->demoAvailable||REGISTERED)&&HasChallengeRequirements(car->challengeRequirements,gConfig->challengeData))||mode==kCarSelectionEnemyMode;	
	gTextOpacity=1;
}

void CarSelectionDrawColors(tFileRef carRef,int color)
{
	glPushAttrib(GL_COLOR_BUFFER_BIT+GL_CURRENT_BIT);
	glDisable(GL_LIGHTING);
	tCarDefinition *car=(tCarDefinition*)FileGetParsedDataPtr(carRef,kParserTypeCarDesc,sizeof(tCarDefinition));

	glLoadIdentity();
	glTranslatef(0.0f,0.0f,-1.0f);
	TexturesSelectTex(FileGetReference("colorbevel.pct"));
	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_S,GL_CLAMP_TO_EDGE);
	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_T,GL_CLAMP_TO_EDGE);
	glEnable(GL_BLEND);
	glBlendFunc(GL_SRC_ALPHA,GL_ONE_MINUS_SRC_ALPHA);
	
	for(int i=0;i<car->numColors;i++)
	{
		glColor3fv(&(car->colors[i].x));
		glBegin(GL_TRIANGLE_STRIP);								
		glTexCoord2d(1,1); glVertex2f(kColorsXPos+kColorsWidth,kColorsYPos-kColorsHeight/2-i*kColorsOffset);		
		glTexCoord2d(1,0); glVertex2f(kColorsXPos+kColorsWidth,kColorsYPos+kColorsHeight/2-i*kColorsOffset); 	
		glTexCoord2d(0,1); glVertex2f(kColorsXPos,kColorsYPos-kColorsHeight/2-i*kColorsOffset); 	
		glTexCoord2d(0,0); glVertex2f(kColorsXPos,kColorsYPos+kColorsHeight/2-i*kColorsOffset); 		
		glEnd();					
	}
	if(car->numColors)
	{
		glColor3f(1,1,1);
		TexturesSelectTex(FileGetReference("menupointer.tif"));
		glBegin(GL_TRIANGLE_STRIP);								
		glTexCoord2d(1,1); glVertex2f(kColorsXPos-kColorsWidth+kPointerSize,kColorsYPos-kPointerSize/2-color*kColorsOffset);		
		glTexCoord2d(1,0); glVertex2f(kColorsXPos-kColorsWidth+kPointerSize,kColorsYPos+kPointerSize/2-color*kColorsOffset); 	
		glTexCoord2d(0,1); glVertex2f(kColorsXPos-kColorsWidth,kColorsYPos-kPointerSize/2-color*kColorsOffset); 	
		glTexCoord2d(0,0); glVertex2f(kColorsXPos-kColorsWidth,kColorsYPos+kPointerSize/2-color*kColorsOffset); 		
		glEnd();					
	}

	glPopAttrib();

}


void InterfaceCarSelectionRenderLoadScreen(float time,int carRef,int mode,int addOns,int color,int drawColor,int *demoAvailable,float carFade,float totalFade)
{
	glPushMatrix();
	glPushAttrib(GL_LIGHTING_BIT);
	glDisable(GL_LIGHTING);
	glClear(GL_COLOR_BUFFER_BIT+GL_DEPTH_BUFFER_BIT);

	CarSelectionDrawBackground(mode);

	CarSelectionDrawSpecs(carRef,mode,demoAvailable);

	InterfaceDrawBackgroundFade(carFade,false);	

	TextPrintfToBufferFormated(Vector(0.2,0.1),0.05,kTextAlignMiddle,"Loading...");

	CarSelectionDrawColors(carRef,color);

	TextPrintBuffer(1,false);
	TextClearBuffer();

	InterfaceDrawBackgroundFade(totalFade,false);	
		
	ScreenBlit();
	glPopAttrib();
	glPopMatrix();
}

void InterfaceCarSelectionRender(float time,int carRef,int mode,int addOns,int color,int drawColor,int *demoAvailable,float carFade,float totalFade)
{
	glPushMatrix();
	glPushAttrib(GL_LIGHTING_BIT);
	glEnable(GL_LIGHTING);

	glClear(GL_COLOR_BUFFER_BIT+GL_DEPTH_BUFFER_BIT);

	CarSelectionDrawBackground(mode);
	/*tInterfaceMenuDescribtion menu;
	InterfaceInitMenu(&menu,0,"");

	InterfaceRenderReplay(NULL,0,&menu);*/

	if(carRef!=-1)
	{
		if(carFade<1)
			CarSelectionDrawCar(carRef,time,addOns,drawColor);
		CarSelectionDrawSpecs(carRef,mode,demoAvailable);
	}

	InterfaceDrawBackgroundFade(carFade,false);	

	if(carRef!=-1&&mode!=kCarSelectionDisplayMode)
		CarSelectionDrawColors(carRef,color);
							
	TextPrintBuffer(1,false);
	TextClearBuffer();

	InterfaceDrawBackgroundFade(totalFade,false);	
		
	ScreenBlit();
	glPopAttrib();
	glPopMatrix();
}

int InterfaceCarSelectionInput(int carCount,int *selection,int mode,int *color,int demoAvailable,int maxColors)
{
	int key;
	GetKeyInput(&key);
	
	switch(key)
	{
		case kInterfaceKeyLeft:
			(*selection)--;
			*color=0;
			PlayInterfaceSound(FileGetReference("menu.wav"));
			break;
		case kInterfaceKeyRight:
			(*selection)++;
			*color=0;
			PlayInterfaceSound(FileGetReference("menu.wav"));
			break;
			
		case kInterfaceKeyUp:
			(*color)--;
			if(*color<0)
				*color=maxColors-1;
			if(*color<0)
				*color=0;
			PlayInterfaceSound(FileGetReference("menu.wav"));
			break;
			
		case kInterfaceKeyDown:
			*color=(*color+1)%maxColors;
			if(*color>=maxColors)
				*color=0;
			PlayInterfaceSound(FileGetReference("menu.wav"));
			break;
			
		case kInterfaceKeyEsc:
		case kInterfaceKeyDelete:
			PlayInterfaceSound(FileGetReference("esc.wav"));
			*selection=-2;
			return true;
	
		case kInterfaceKeyReturn:
		case kInterfaceKeyEnter:
			if(demoAvailable)
			{
				PlayInterfaceSound(FileGetReference("select.wav"));
				return true;	
			}
			break;
	}		
	return false;
}

void SelectNextCar(tFileRef *car,UInt8 *color,int onlyAvailable)
{
	int availableCars[kMaxCars];
	int carCount;
	int selection=0;
	GetAvailableCars(availableCars,&carCount,false,onlyAvailable);
	
	for(int i=0;i<carCount;i++)
		if(*car==availableCars[i])
			selection=i;

	tCarDefinition *c;
	do{
		selection=(selection+1)%carCount;
		c=(tCarDefinition*)FileGetParsedDataPtr(availableCars[selection],kParserTypeCarDesc,sizeof(tCarDefinition));
		if(color)
			if(c->numColors)
				*color=RandomInt(0,c->numColors);
			else
				*color=0;
	}while(!c->demoAvailable&&!REGISTERED);
	
	*car=availableCars[selection];
//	gConfig->lastCar=*car;
}

void SelectPrevCar(tFileRef *car,UInt8 *color,int onlyAvailable)
{
	int availableCars[kMaxCars];
	int carCount;
	int selection=0;
	GetAvailableCars(availableCars,&carCount,false,onlyAvailable);
	
	for(int i=0;i<carCount;i++)
		if(*car==availableCars[i])
			selection=i;

	tCarDefinition *c;
	do{
		selection--;
		if(selection<0)
			selection+=carCount;
		c=(tCarDefinition*)FileGetParsedDataPtr(availableCars[selection],kParserTypeCarDesc,sizeof(tCarDefinition));
		if(color)
			if(c->numColors)
				*color=RandomInt(0,c->numColors);
			else
				*color=0;
	}while(!c->demoAvailable&&!REGISTERED);
	
	*car=availableCars[selection];
//	gConfig->lastCar=*car;
}

void SetupLighting();

int InterfaceCarSelection(tFileRef *car,int mode,UInt8 *pColor,int (*Callback)(void*),void *userData,int *callbackResponse)
{
	int availableCars[kMaxCars];
	int selection=0;
	int carCount;
	int color=*pColor;
	int oldColor=*pColor;
	int demoAvailable=true;
	tFileRef curCar;
	float carFade=0;
			
	GetAvailableCars(availableCars,&carCount,false,false);
	for(int i=0;i<carCount;i++)
		if(*car==availableCars[i])
			selection=i;

	tEnvironment *oldEnv=gEnvironment;
	tMapEnv* oldMapEnv=gMapEnv;
	gMapEnv=NULL;
	LoadEnvironment(FileGetReference("showroom.senv"));
	SetupLighting();
	glDisable(GL_LIGHTING);
//workaround for drawing bug
	InterfaceCarSelectionRender(0,FileGetReference("diablo.car"),mode,0,color,oldColor,&demoAvailable,1,1);
	
	float startTime=TimeGetSeconds();
	float curTime=0;

	int maxColors=1;
	
	curCar=selection==-1?-1:availableCars[selection];

	do{
		selection=selection%carCount;
		if(selection<0) selection+=carCount;		
		float dt=curTime;
		curTime=TimeGetSeconds()-startTime;
		dt=curTime-dt;

		if((curCar!=(selection==-1?-1:availableCars[selection]))||color!=oldColor)
		{
			if(selection!=-1)
			{
				tCarDefinition *car=(tCarDefinition*)FileGetParsedDataPtr(availableCars[selection],kParserTypeCarDesc,sizeof(tCarDefinition));			
				int colorLoaded=false;
				if(car->numColors>0)
					colorLoaded=car->colorLoaded[color];
				if(carFade>=3||(colorLoaded&&carFade>=1))
				{
					curCar=availableCars[selection];
					oldColor=color;
				}
				else
					carFade+=dt/0.2;
			}
			else if(carFade>=1)
			{
				curCar=-1;
				oldColor=color;
			}
			else
				carFade+=dt/0.2;	
		}
		else
		{
			carFade-=dt/0.2;
			if(carFade<1&&carFade+dt/0.2>=1&&curCar!=-1)
			{
				tCarDefinition *car=(tCarDefinition*)FileGetParsedDataPtr(availableCars[selection],kParserTypeCarDesc,sizeof(tCarDefinition));			
				int colorLoaded=false;
				if(car->numColors>0)
					colorLoaded=car->colorLoaded[color];
				if(!colorLoaded)
				{
					InterfaceCarSelectionRenderLoadScreen(curTime,curCar,mode,0,color,oldColor,&demoAvailable,carFade,1-(curTime/0.2));
					InterfaceCarSelectionRender(curTime,curCar,mode,0,color,oldColor,&demoAvailable,carFade,1-(curTime/0.2));
					FlushKeys();
					curTime=TimeGetSeconds()-startTime;
				}
			}
		}
		if(carFade<0)carFade=0;
		if(carFade>=1)
			InterfaceCarSelectionRender(curTime,selection==-1?-1:availableCars[selection],mode,0,color,oldColor,&demoAvailable,carFade,1-(curTime/0.2));	
		else				
			InterfaceCarSelectionRender(curTime,curCar,mode,0,color,oldColor,&demoAvailable,carFade,1-(curTime/0.2));	
		SystemPoll(false);	
		if(Callback)
		{
			*callbackResponse=Callback(userData);
			if(*callbackResponse!=-1)
			{
				gEnvironment=oldEnv;
				gMapEnv=oldMapEnv;
				return false;
			}
		}
		if(selection!=-1)
		{
			tCarDefinition *car=(tCarDefinition*)FileGetParsedDataPtr(availableCars[selection],kParserTypeCarDesc,sizeof(tCarDefinition));
			maxColors=car->numColors;
			if(maxColors==1)maxColors=1;
		}
		else
			maxColors=1;
	}
	while(!InterfaceCarSelectionInput(carCount,&selection,mode,&color,demoAvailable,maxColors));
	glEnable(GL_LIGHTING);
	gEnvironment=oldEnv;
	gMapEnv=oldMapEnv;
	
	float endTime=curTime+0.2;
	while(curTime<endTime)
	{
		curTime=TimeGetSeconds()-startTime;
		InterfaceCarSelectionRender(curTime,curCar,mode,0,color,oldColor,&demoAvailable,carFade,1-((endTime-curTime)/0.2));			
	}
	if(selection!=-2)
	{
		*car=availableCars[selection];
		*pColor=color;
	}
	return selection!=-2;
}

enum{
	kOpponentNumOpponents,
	kOpponent1,
	kOpponent2,
	kOpponent3,
	kOpponent4,
	kOpponent5,
	kOpponentOK,
	kOpponentNumOptions
};

int InterfaceSelectOpponentCars(int *numOpponents,tFileRef *opponents,UInt8 *colors,int (*Callback)(void*),void *userData)
{
	int setNumOpponents=gConfig->numEnemies;
	int callBackResponse;
	tInterfaceMenuDescribtion menu;
	InterfaceInitMenu(&menu,kOpponentNumOptions,"Select Opponents");

	int availableCars[kMaxCars];
	int carCount;
	GetAvailableCars(availableCars,&carCount,false,false);

//	for(int i=0;i<menu.numItems;i++)
//		strcpy(menu.items[i].describtion,itemStrings[i]);


	strcpy(menu.items[kOpponentOK].label,"Accept");
	menu.items[kOpponentOK-1].lineSpacing*=1.5;
	menu.items[kOpponent1-1].lineSpacing*=1.5;
	menu.TimerCallback=Callback;
	menu.userData=userData;
	
	for(int i=kOpponentNumOpponents;i<=kOpponent5;i++)
		menu.items[i].flags|=kInterfaceMenuItemArrowInput;

	for(int i=0;i<5;i++)
	{
		opponents[i]=gConfig->opponentCars[i];
		colors[i]=gConfig->opponentColors[i];
		if(opponents[i]==-1||opponents[i]==0)
		{
			opponents[i]=availableCars[0];
			colors[i]=0;
		}
	}
	
	int exit=false;
	int zoom=false;
	do{
		sprintf(menu.items[kOpponentNumOpponents].label,"Number of Opponents: \255#a\255%d",setNumOpponents);
		for(int i=kOpponent1;i<=kOpponent5;i++)
		{
			if(i-kOpponent1<setNumOpponents)
				menu.items[i].flags&=~kInterfaceMenuItemDisabled;
			else
				menu.items[i].flags|=kInterfaceMenuItemDisabled;
			tCarDefinition *c=(tCarDefinition*)FileGetParsedDataPtr(opponents[i-kOpponent1],kParserTypeCarDesc,sizeof(tCarDefinition));			
			sprintf(menu.items[i].label,"Opponent %d: \255#a\255%s",i-kOpponent1+1,c->carName);
		}	
		int sel=InterfaceGetUserMenuSelection(&menu);
		switch(menu.initialSelection=(sel&kInterfaceMenuItemMask))
		{
			case kOpponentNumOpponents:
				if(sel&kInterfaceMenuRightArrow){
					if(setNumOpponents<5)
						setNumOpponents++;
				}
				else if(sel&kInterfaceMenuLeftArrow){
					if(setNumOpponents>0)
						setNumOpponents--;
				}
				else
					exit=true;
				break;
			case kOpponent1:
			case kOpponent2:
			case kOpponent3:
			case kOpponent4:
			case kOpponent5:
				if(sel&kInterfaceMenuLeftArrow)
					SelectPrevCar(&opponents[menu.initialSelection-kOpponent1],&colors[menu.initialSelection-kOpponent1],false);
				else if(sel&kInterfaceMenuRightArrow)
					SelectNextCar(&opponents[menu.initialSelection-kOpponent1],&colors[menu.initialSelection-kOpponent1],false);
				else
					InterfaceCarSelection(&opponents[menu.initialSelection-kOpponent1],kCarSelectionEnemyMode,&colors[menu.initialSelection-kOpponent1],Callback,userData,&callBackResponse);
				break;
			case kOpponentOK:
			case kInterfaceMenuEsc:
				exit=true;
				break;

		}
	}while(!exit);
	for(int i=0;i<5;i++)
	{
		gConfig->opponentCars[i]=opponents[i];
		gConfig->opponentColors[i]=colors[i];
	}
	*numOpponents=gConfig->numEnemies=setNumOpponents;
	InterfaceDisposeMenu(&menu);	
	return menu.initialSelection!=kInterfaceMenuEsc;
}